package com.xylink.config;


import java.util.HashMap;
import java.util.Map;

/**
 * @author: yxk
 * @Date: 2022/7/28-- 7:38 PM:
 * @Description: 签名版本枚举
 */
public enum SignatureVersion {

    Signature_1_0("1.0"),
    Signature_2_0("2.0"),
    Signature_2_0_1("2.0.1"),
    Signature_3_0("3.0")
    //所有版本"必须"按照从小到大顺序排列，
    // 版本所支持的特性不会向前兼容，比如3.0及以后支持的从uaa获取时间戳功能将不会在3.0之前支持
    ;

    private static Map<SignatureVersion, Integer> enumMap = new HashMap<>();
    static {
        for (SignatureVersion value : SignatureVersion.values()) {
            enumMap.put(value, value.ordinal());
        }
    }


    SignatureVersion(String version) {
        this.version = version;
    }

    private String version;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    //按照序号进行处理，如果大于1.0就认为是新的签名方式
    public boolean isNewSign(){
        Integer ordinal1_0 = enumMap.get(Signature_1_0);
        return this.ordinal() > ordinal1_0;
    }

    //3.0以后就认为是时间安全的版本，会去uaa请求时间戳
    public boolean isTimeSafeVersion(){
        Integer ordinal3_0 = enumMap.get(Signature_3_0);
        return this.ordinal() >= ordinal3_0;
    }

}
