package com.xylink.util;

import com.xylink.config.SDKConfigMgr;
import com.xylink.log.XyLinkLog;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.RandomStringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.KeyFactory;
import java.security.Security;
import java.security.spec.X509EncodedKeySpec;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Base64;
import java.util.Objects;
import java.util.regex.Pattern;

public class AuthcodeUtil {


    public static final String SEPARATOR = ",";

    private static final Logger log = LoggerFactory.getLogger(AuthcodeUtil.class);

    // private static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
    public static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
    public static Pattern extUserIdMatcher = Pattern.compile("^[0-9a-zA-Z_-]{0,50}$");
    public static final String KEY_ALGORITHM = "RSA";
    private static XyLinkLog logger = SDKConfigMgr.getXyLinkLog();
    public static final String ALGORITHM = "SM4/CBC/PKCS5Padding";
    public static final String ALGORITHM_SM4_CBC_PKCS7PADDING = "SM4/CBC/PKCS7Padding";

    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    /**
     * RSA加密方式获取authcode
     * @param externalUserId
     * @param pubKey
     * @return authCode usable in nowtime +/- 2 minutes （避免云机器时间差过大）
     * @throws Exception
     */
    public static String generateAuthCodeByRSA(String externalUserId, String pubKey) {
        if (!matchExtUserId(externalUserId)){
            logger.info(externalUserId + " externalUserId formatter error");
            return null;
        }
        String ramdom = RandomStringUtils.random(11, true, false);
        LocalDateTime dateTime = LocalDateTime.now();
        String timeStr = dateTime.format(timeStrFormatter);
        String content = externalUserId + SEPARATOR + timeStr+ SEPARATOR+ramdom;
        logger.info("加密前："+content);
        return encryptByPubKey4String(content, pubKey);
    }

    public static byte[] rsaEncryptByPublicKey(byte[] data, String publicKey)
            throws Exception {
        //得到公钥
        byte[] keyBytes = Base64.getDecoder().decode(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        // 对数据加密
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicK);
        return cipher.doFinal(data);
    }

    public static byte[] encryptByPubKey4Byte(String data, String publicKey) {
        if (data == null || data.equals("")) {
            return null;
        }
        byte[] publicEncrypData = null;
        try {
            //1、data base64 encode
            String dataBase64Encoded = Base64.getEncoder().encodeToString(data.getBytes(StandardCharsets.UTF_8));
            logger.info("rsaencrypt dataBase64Encoded "+dataBase64Encoded);
            //2、encrypt
            publicEncrypData = rsaEncryptByPublicKey(dataBase64Encoded.getBytes(StandardCharsets.UTF_8), publicKey);
        } catch (Exception e) {
            logger.error("rsaEncryptByPubKey4Byte error "+e);
        }
        return publicEncrypData;
    }

    public static String encryptByPubKey4String(String data, String publicKey) {
        byte[] bytes = encryptByPubKey4Byte(data, publicKey);
        return Hex.encodeHexString(bytes);
    }

    public static boolean matchExtUserId(String extUserId){
        logger.info(extUserId.length()+" extUserId ->"+extUserId);
        return extUserIdMatcher.matcher(extUserId).find();
    }


    /**
     * 所使用cs5，秘钥向量为字符串形式，长度为16
     * @param key
     * @param content
     * @return
     */
    public static String SM4Encrypt(String key, String content){
       return SM4Encrypt(ALGORITHM, key, content);
    }

    /**
     * 自定义算法，秘钥向量为字符串形式，长度为16
     * @param key
     * @param content
     * @return
     */
    public static String SM4Encrypt(String algorithm, String key, String content) {
        byte[] bytes = key.getBytes(StandardCharsets.UTF_8);
        byte[] keyBytes = getBytes(bytes);
        SecretKeySpec secretKey = new SecretKeySpec(keyBytes, algorithm);
        Arrays.sort(keyBytes);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(keyBytes);
        return SM4Encrypt(algorithm, secretKey, content, ivParameterSpec);
    }

    /**
     * 自定义算法、向量、秘钥
     * @param algorithm
     * @param secretKey
     * @param content
     * @param ivParameterSpec
     * @return
     */
    public static String SM4Encrypt(String algorithm, SecretKeySpec secretKey, String content, IvParameterSpec ivParameterSpec){
        try {
            Cipher instance = Cipher.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME);
            instance.init(Cipher.ENCRYPT_MODE, secretKey, ivParameterSpec);
            byte[] contentBytes = content.getBytes(StandardCharsets.UTF_8);
            byte[] bytes1 = instance.doFinal(contentBytes, 0, contentBytes.length);
            return org.bouncycastle.util.encoders.Hex.toHexString(bytes1);
        } catch (Exception e) {
            logger.error(" ",e);
            return null;
        }
    }

    /**
     * 所使用cs5，秘钥向量为字符串形式，长度为16
     * @param key
     * @param content
     * @return
     */
    public static String SM4DeEnt(String key, String content){
       return SM4DeEnt(ALGORITHM, key, content);
    }

    /**
     * 自定义算法，秘钥向量为字符串形式，长度为16
     * @param key
     * @param content
     * @return
     */
    public static String SM4DeEnt(String algorithm, String key, String content){
        byte[] bytes = key.getBytes(StandardCharsets.UTF_8);
        byte[] keyBytes = getBytes(bytes);
        SecretKeySpec secretKey = new SecretKeySpec(keyBytes, algorithm);
        Arrays.sort(keyBytes);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(keyBytes);
        return SM4DeEnt(algorithm, secretKey, content, ivParameterSpec);
    }

    /**
     * 自定义算法、向量、秘钥
     * @param algorithm
     * @param secretKey
     * @param content
     * @param ivParameterSpec
     * @return
     */
    public static String SM4DeEnt(String algorithm, SecretKeySpec secretKey, String content, IvParameterSpec ivParameterSpec){
        try {
            Cipher instance = Cipher.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME);
            instance.init(Cipher.DECRYPT_MODE, secretKey, ivParameterSpec);
            byte[] decode = org.bouncycastle.util.encoders.Hex.decode(content);
            byte[] bytes1 = instance.doFinal(decode, 0, decode.length);
            return new String(bytes1);
        } catch (Exception e) {
            logger.error(" ",e);
            return null;
        }
    }

    private static byte[] getBytes(byte[] keyBytes) {
        byte[] reslutBytes = new byte[16];
        System.arraycopy(keyBytes, 0, reslutBytes, 0, Math.min(keyBytes.length, 16));
        return reslutBytes;
    }

    /**
     * 基于SM4CBCPKCS5算法的 iv基于key生成的的加密,秘钥长度16字符串
     * @param externalUserId
     * @param key
     * @return
     */
    public static String generateAuthCodeBySM4(String externalUserId, String key) {
        String random =  RandomStringUtils.random(10,true,false);
        String content = externalUserId +";"+ System.currentTimeMillis() + ";"+ random;
        return SM4Encrypt(key, content);
    }

    /**
     * 基于SM4CBCPKCS7算法的 iv基于key生成的的加密,秘钥长度16字符串
     * @param externalUserId externalUserId
     * @param key 加密秘钥
     * @return authcode
     */
    public static String generateAuthCodeBySM4CBCPKCS7(String externalUserId, String key) {
        String random =  RandomStringUtils.random(10,true,false);
        String content = externalUserId +";"+ System.currentTimeMillis() + ";"+ random;
        return SM4Encrypt(ALGORITHM_SM4_CBC_PKCS7PADDING, key, content);
    }

    /**
     * 基于SM4CBCPKCS7算法的 iv为0的加密
     * @param externalUserId externalUserId
     * @param hexKey 加密秘钥
     * @return authcode
     */
    public static String generateAuthCodeBySM4CBCPKCS7WithIvZerosHexKey(String externalUserId, String hexKey) {
        String random =  RandomStringUtils.random(10,true,false);
        String content = externalUserId +";"+ System.currentTimeMillis() + ";"+ random;
        SecretKeySpec secretKey = new SecretKeySpec(Objects.requireNonNull(HexUtil.decodeHex(hexKey)), ALGORITHM_SM4_CBC_PKCS7PADDING);
        byte[] ivBytes = new byte[16];
        Arrays.fill(ivBytes, (byte) 0x00);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(ivBytes);
        return SM4Encrypt(ALGORITHM_SM4_CBC_PKCS7PADDING, secretKey, content, ivParameterSpec);
    }

    public static String deEncryptAuthCodeBySM4CBCPKCS7WithIvZerosHexKey(String encrypet, String hexKey) {
        SecretKeySpec secretKey = new SecretKeySpec(Objects.requireNonNull(HexUtil.decodeHex(hexKey)), ALGORITHM_SM4_CBC_PKCS7PADDING);
        byte[] ivBytes = new byte[16];
        Arrays.fill(ivBytes, (byte) 0x00);
        IvParameterSpec ivParameterSpec = new IvParameterSpec(ivBytes);
        return SM4DeEnt(ALGORITHM_SM4_CBC_PKCS7PADDING, secretKey, encrypet, ivParameterSpec);
    }

    public static void main(String[] args) throws Exception {
        String hexKey = args[0];
        String encrypet = args[1];
        System.out.println("hexKey:" + hexKey + ", encrypet:" + encrypet + ", env variables:{}" + Arrays.toString(args));

        String string = deEncryptAuthCodeBySM4CBCPKCS7WithIvZerosHexKey(encrypet, hexKey);
        System.out.println(string);
    }
}

