package com.xylink.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xylink.config.SDKConfigMgr;
import com.xylink.log.XyLinkLog;
import org.bouncycastle.crypto.digests.SM3Digest;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import java.security.Security;

public class EncryptUtil {

    private static XyLinkLog logger = SDKConfigMgr.getXyLinkLog();

    private static final ObjectMapper mapper = new ObjectMapper();

    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    /**
      *国密SM3加盐hash
      *@param key
      *@param value
      *@return
     */
    private static String sm3SaltedHashValue(String key, String value) {
        //加盐
        byte[] passwdbyte = arraycat(key.getBytes(),value.getBytes());
        //SM3计算
        SM3Digest mdDigest=new SM3Digest();
        mdDigest.update(passwdbyte,0,passwdbyte.length);
        byte[] result=new byte[mdDigest.getDigestSize()];
        mdDigest.doFinal(result, 0);
        return HexUtil.encodeHexStr(result);
    }

    /**
     * 拼接buf1和buf2数组
     */
    private static byte[] arraycat(byte[] buf1,byte[] buf2) {
        byte[] bufret=null;
        int len1=0;
        int len2=0;
        if(buf1!=null) {
            len1=buf1.length;
        }
        if(buf2!=null) {
            len2=buf2.length;
        }
        if(len1+len2>0) {
            bufret=new byte[len1+len2];
        }
        if(len1>0) {
            System.arraycopy(buf1,0,bufret,0,len1);
        }
        if(len2>0) {
            System.arraycopy(buf2,0,bufret,len1,len2);
        }
        return bufret;
    }

    /**
     * 计算签名
     *
     * @param token
     * @param content 内容,为JSON字符串，否则可能报错
     * @return
     */
    public static String calcSign(String token, String content) throws JsonProcessingException {
        //签名计算
        Object socketMessage = mapper.readValue(content, Object.class);
        logger.info(socketMessage.toString());
        String content1 = mapper.writeValueAsString(socketMessage);
        content1 = content1.length() > 100 ? content1.substring(0, 100) : content1;
        String encrypt = EncryptUtil.sm3SaltedHashValue(token, content1);
        return encrypt.substring(0, 30);
    }
}